"use strict";

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

/*
 * @Author: spark
 * @Date: 2021-01-28 17:03:50
 * @LastEditTime: 2021-02-04 10:32:27
 * @LastEditors: spark
 * @Description: 趣看天下前端开发
 * @email: spark.xiaoyu@qq.com
 */
var JavaScriptObfuscator = require("javascript-obfuscator");

var Spinner = require("./utils/spinner");

var pluginName = "JavaScriptObfuscatorPlugin";

function JavaScriptObfuscatorPlugin(options) {
  this.options = _extends({
    compact: true,
    controlFlowFlattening: true,
    controlFlowFlatteningThreshold: 0.75,
    deadCodeInjection: true,
    deadCodeInjectionThreshold: 0.4,
    debugProtection: false,
    debugProtectionInterval: false,
    disableConsoleOutput: true,
    identifierNamesGenerator: "hexadecimal",
    log: false,
    numbersToExpressions: true,
    renameGlobals: false,
    rotateStringArray: true,
    selfDefending: false,
    shuffleStringArray: true,
    simplify: true,
    splitStrings: true,
    splitStringsChunkLength: 10,
    stringArray: true,
    stringArrayEncoding: ["base64"],
    stringArrayIndexShift: true,
    stringArrayWrappersCount: 2,
    stringArrayWrappersChainedCalls: true,
    stringArrayWrappersParametersMaxCount: 4,
    stringArrayWrappersType: "function",
    stringArrayThreshold: 0.75,
    transformObjectKeys: true,
    unicodeEscapeSequence: false
  }, options || {});
}

JavaScriptObfuscatorPlugin.prototype.apply = function (compiler) {
  //注册事件回调有三个方法： tap、tapAsync 和 tapPromise，其中 tapAsync 和 tapPromise 不能用于 Sync 开头的钩子类，强行使用会报错
  // console.log(compiler.hooks);
  var _this = this;

  var progress = new Spinner();
  compiler.hooks.run.tap(pluginName, function () {
    progress.start("AST加密构建过程开始！", "blue");
  });
  compiler.hooks.emit.tapAsync(pluginName, function (compilation, callback) {
    for (var filename in compilation.assets) {
      // console.log(compilation.assets[filename].source())
      var _flag = compilation.assets[filename].source();

      if (/\.js$/gi.test(filename)) {
        (function () {
          var content = JavaScriptObfuscator.obfuscate(_flag, _this.options).getObfuscatedCode();

          if (content) {
            compilation.assets[filename] = {
              source: function source() {
                return content;
              },
              size: function size() {
                return content.length;
              }
            };
            progress.setText(filename + ": AST加密完成");
          }
        })();
      }
    }

    progress.succeed("加密进程结束");
    callback();
  });
};

module.exports = JavaScriptObfuscatorPlugin;